/*
   backend interface

   Copyright (C) 2003 Lars Lindner <lars.lindner@gmx.net>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.
   
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.
   
   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/

#ifndef _BACKEND_H
#define _BACKEND_H

#include <gtk/gtk.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>

/* constants for attributes in feedstore */
#define FS_TITLE	0
#define FS_URL		1
#define FS_STATE	2
#define FS_CP		3

/* constants for attributes in itemstore */
#define IS_TITLE	0
#define IS_STATE	1
#define IS_PTR		2
#define IS_TIME		3

/* to connect the data to any views... */
GtkTreeStore * getFeedStore(void);
GtkTreeStore * getItemStore(void);

/* data definitions for channel and item storage */
typedef struct item {
	gchar		*title;		/* standard namespace infos */
	gchar		*description;
	gchar		*link;
	gchar		*image;
	
	GHashTable	*nsinfos;	/* list to store pointers to namespace
					   specific informations */	
	gboolean	read;
	gchar		*time;
	struct item	*next;
} item, *itemPtr;

typedef struct channel {
	gchar		*title;		/* standard namespace infos */
	gchar		*description;
	gchar		*link;
	gchar		*image;

	GHashTable	*nsinfos;	/* list to store pointers to namespace
					   specific informations */
					   
	itemPtr 	items;		/* the item list */
	
	// FIXME: outsource properties to a document structure?
	gchar		*time;		/* last update time */
	gchar		*usertitle;	/* feed title may be modified by user */	
	gchar 		*encoding;	/* xml encoding */
	gchar 		*source;	/* source url */
	gchar		*feedkey;	/* configuration storage key */
	gboolean	available;	/* flag to signalize load/update errors */
} channel, *channelPtr;

/* -------------------------------------------------------- */
/* interface definitions for namespace handler              */
/* -------------------------------------------------------- */

/* definition of various namespace tag handlers */
typedef void	(*parseChannelTagFunc)	(channelPtr cp, xmlDocPtr doc, xmlNodePtr cur);
typedef void	(*parseItemTagFunc)	(itemPtr ip, xmlDocPtr doc, xmlNodePtr cur);

/* handler called during HTML output generation to display
   namespace specific information (e.g. <dc:creator> the 
   handler could return HTML like: "<p>author: Mr. X</a>" */
typedef gchar *	(*outputFunc)	(gpointer obj, gpointer htmlStream);

/* struct used to register RDF namespace handler */
typedef struct nsHandler {
	parseItemTagFunc	parseItemTag;
	parseChannelTagFunc	parseChannelTag;
		
	outputFunc	doItemHeaderOutput;
	outputFunc	doItemFooterOutput;	
	outputFunc	doChannelHeaderOutput;
	outputFunc	doChannelFooterOutput;	
} nsHandler;

/* -------------------------------------------------------- */
/* functions to access properties of actually selected feed */
/* -------------------------------------------------------- */

/* retrieves feed from url and parse it, newflag is true
   for new feeds and false for updates, if createflag is 
   specified the feed is added to the gconf feed list */
int	newFeed(gchar *feedkey, gchar *url, gboolean newflag, gboolean saveflag);
void 	removeFeed(gpointer cp);

void 	setFeedTitle(gpointer cp, gchar *feedname);
void 	setFeedURL(gpointer cp, gchar *feedurl);

void	moveUpFeedPosition(gpointer cp);
void	moveDownFeedPosition(gpointer cp);

GHashTable * getFeedNSHandler(gpointer cp);	/* returns the Hashtable with the namespace I/O-handlers */
gchar *	getFeedURL(gpointer cp);
gchar *	getFeedSource(gpointer cp);
gchar * getFeedImage(gpointer cp);
gchar * getFeedDescription(gpointer cp);
gchar * getFeedTitle(gpointer cp);		/* returns the user defined feed title */
gchar *	getDefaultFeedTitle(gpointer cp);	/* returns the title defined by the feed */

/* -------------------------------------------------------- */
/* functions to access properties of actually selected item */
/* -------------------------------------------------------- */

gboolean getItemReadStatus(gpointer ip);
gchar * getItemTitle(gpointer ip);
gchar *	getItemLink(gpointer ip);
gchar * getItemContent(gpointer ip);

void	loadItems(gpointer cp);
void	clearItems();
void	markItemAsRead(gpointer ip, GtkTreeIter *iter);	// FIXME: remove Iter!

#endif
